package main

import (
	"log"

	"google.golang.org/grpc"

	"github.com/valyala/fasthttp"

	backGRpc "git.loafle.net/overflow/overflow_api_server/golang"
	"git.loafle.net/overflow/overflow_gateway_web/handler/file"
	"git.loafle.net/overflow/overflow_gateway_web/handler/web"
	gws "git.loafle.net/overflow/overflow_gateway_websocket"
	grpcPool "git.loafle.net/overflow/overflow_grpc_pool"
)

func main() {
	o := &gws.ServerOptions{
		OnConnection:   onConnection,
		OnDisconnected: onDisconnected,
		OnCheckOrigin:  onCheckOrigin,
	}
	s := gws.NewServer(o)

	bo := &grpcPool.Options{
		MaxIdle:     1,
		MaxCapacity: 3,

		Creators: func() (*grpc.ClientConn, interface{}, error) {
			var err error
			conn, err := grpc.Dial(":50006", grpc.WithInsecure())
			if nil != err {
				return nil, nil, err
			}
			c := backGRpc.NewOverflowApiServerClient(conn)
			return conn, c, nil
		},
	}
	bPool, err := grpcPool.New(bo)
	if nil != err {
		log.Panic("Cannot create Pool of GRPC")
	}

	wh := web.New(bPool)
	fh := file.New()

	s.HandleSocket("/web", wh.GetSocketOption())
	s.HandleSocket("/file", fh.GetSocketOption())

	s.ListenAndServe(":19090")
}

func onCheckOrigin(ctx *fasthttp.RequestCtx) bool {
	if origin := string(ctx.Request.Header.Peek("Origin")); origin != "" {
		ctx.Response.Header.Set("Access-Control-Allow-Origin", origin)
		if string(ctx.Method()) == "OPTIONS" && string(ctx.Request.Header.Peek("Access-Control-Request-Method")) != "" {
			ctx.Response.Header.Set("Access-Control-Allow-Headers", "Content-Type, Accept")
			ctx.Response.Header.Set("Access-Control-Allow-Methods", "GET, HEAD, POST, PUT, DELETE")
		}
	}
	return true
}

func onConnection(soc gws.Socket) {
	log.Printf("connect: path: %s, id:%s\n", soc.Path(), soc.ID())
}

func onDisconnected(soc gws.Socket) {
	log.Printf("disconnect: path: %s, id:%s\n", soc.Path(), soc.ID())
}
